package kz.arta.ext.sap.service;

import kz.arta.ext.sap.util.ImportBean;
import org.apache.commons.io.IOUtils;

import org.jboss.resteasy.plugins.providers.multipart.InputPart;
import org.jboss.resteasy.plugins.providers.multipart.MultipartFormDataInput;
import org.jboss.resteasy.util.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.enterprise.context.RequestScoped;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.*;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.core.Response;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 *
 * REST сервис с методами, которые не требуют авторизации
 */
@Path("/registry")
@RequestScoped
public class UnsecuredProxyService {

    private static final Logger LOGGER = LoggerFactory.getLogger(UnsecuredProxyService.class);

    @GET
    @Path("/test")
    @Produces(MediaType.APPLICATION_JSON + "; charset=utf-8")
    public String test() {
        return "{\"status\":\"working\"}";
    }

    /**
     * Импорт записей в реестр
     * @param req
     * @param input
     * @param formcode - код формы реестра, по которому будет проводиться импорт,
     * @param registryCode - код реестра, в который импортируются записи,
     * @param isActivate - нужно ли активировать импортированные записи
     * @param action - что делать с ссылкой на реестр, если найденно несколько записей реестра с таким значением: "0" - ставить дефолтную запись, "1" - оставлять пустым, "2" - брать первую найденную
     * @param defaultValue - значение по умолчанию для action "0". В формате field_id1:documentID, field_id2:documentID, для всех значений кроме 0 передавать none
     * @param searchString - значение для поиска для ссылок на реестр в формате field_id1:registry_search_field, field_id2:registry_search_field, field_id3:registry_search_field
     * @return
     */
    @POST
    @Consumes(MediaType.MULTIPART_FORM_DATA)
    @Path("/import_file")
    @Produces(MediaType.APPLICATION_JSON + "; charset=utf-8")
    public Response importToRegistry(@Context HttpServletRequest req, MultipartFormDataInput input, @QueryParam("formcode") String formcode,
                                     @QueryParam("registrycode") String registryCode, @QueryParam("activate") Boolean isActivate,
                                     @QueryParam("action") int action, @QueryParam("defaultValue") String defaultValue, @QueryParam("searchString") String searchString){

        String fileName = "";
        String result = "";

        try {

            String auth = req.getHeader("Authorization");
            auth = auth.replace("Basic ", "");
            String decoded = new String(Base64.decode(auth));
            String[] authAr = decoded.split(":");
            String login = authAr[0];
            String password = authAr[1];


            Map<String, List<InputPart>>  uploadForm = input.getFormDataMap();
            List<InputPart> inputParts = uploadForm.get("file");

            for(InputPart inputPart : inputParts) {
                MultivaluedMap<String, String> header = inputPart.getHeaders();
                fileName = getFileName(header);

                InputStream inputStream = inputPart.getBody(InputStream.class, null);
                byte[] bytes = IOUtils.toByteArray(inputStream);

                fileName = "/tmp/crm-import/" + fileName;

                writeFile(bytes, fileName);

                LOGGER.info("File " + fileName + "was written!");

                result = ImportBean.parseFile(fileName, formcode, registryCode, isActivate, login, password, action, defaultValue, searchString);
            }
        } catch (Exception e){
            LOGGER.error(e.getMessage());
            return Response.status(500).entity(e.getMessage()).build();
        }

        return Response.status(200).entity(result).build();

    }


    private String getFileName(MultivaluedMap<String, String> header){
        String[] contentDisposition = header.getFirst("Content-Disposition").split(";");

        for (String filename : contentDisposition) {
            if ((filename.trim().startsWith("filename"))) {

                String[] name = filename.split("=");

                String finalFileName = name[1].trim().replaceAll("\"", "");
                return finalFileName;
            }
        }
        return "unknown";
    }

    //save to the path
    private void writeFile(byte[] content, String filename) throws IOException {

        File file = new File(filename);

        if(!file.exists()){
            file.createNewFile();
        }

        FileOutputStream fop = new FileOutputStream(file);

        fop.write(content);
        fop.flush();
        fop.close();
    }

}
