package kz.arta.ext.sap.util;

import org.joda.time.DateTime;
import org.joda.time.format.*;

import java.util.Date;

public class JodaDateFormat implements DateFormat {
    private String pattern;
    private final DateTimeFormatter formatter;
    private final DateTimeFormatter strictFormatter;

    public JodaDateFormat(String pattern) {
        this.pattern = pattern;
        formatter = new DateTimeFormatterBuilder().appendPattern(pattern).append(new JodaDateTimeConsumeAll()).toFormatter();
        strictFormatter = DateTimeFormat.forPattern(pattern);
    }

    @Override
    public Date parse(String text) throws Exception {
        return doParse(formatter, text);
    }

    @Override
    public Date parseStrict(String text) throws Exception {
        return doParse(strictFormatter, text);
    }

    private Date doParse(DateTimeFormatter formatter, String text) throws Exception {
        DateTime dateTime = formatter.parseDateTime(text);
        return dateTime.toDate();
    }

    @Override
    public String format(Date date) {
        return formatter.print(date.getTime());
    }

    @Override
    public String format(long time) {
        return formatter.print(time);
    }

    @Override
    public String getPattern() {
        return pattern;
    }

    public class JodaDateTimeConsumeAll implements DateTimeParser{
        @Override
        public int estimateParsedLength() {
            return 100;
        }
        @Override
        public int parseInto(DateTimeParserBucket bucket, String text, int position) {
            return text.length(); //consume the whole thing
        }

    }
}
